#! /usr/bin/env python

#######################################
# A script to calculate statistics from
# a text file using NumPy
# Author: <YOUR NAME>
# Email: <YOUR EMAIL>
# Date: DD/MM/YYYY
# Version: 1.0
#######################################

import numpy as np
import scipy as sp
# Import scipy stats functions we need
import scipy.stats as spstats

class CalculateStatistics (object):
    
    def run(self):
        # Set up lists to hold input diameters and heights
        # A seperate list is used for each species
        beechDiameter = list()
        beechHeight = list()
        ashDiameter = list()
        ashHeight = list()
        birchDiameter = list()
        birchHeight = list()
        oakDiameter = list()
        oakHeight = list()
        sycamoreDiameter = list()
        sycamoreHeight = list()
        otherDiameter = list()
        otherHeight = list()
        
        # Open input and output files
        inFileName = 'PenglaisWoodsData.csv'
        outFileName = 'PenglaisWoodsStats.csv'
        inFile = open(inFileName, 'r')
        outFile = open(outFileName,'w')
        
        # Iterate through the input file and save diameter and height
        # into lists, based on species
        header = True 
        for eachLine in inFile:
            if header: # Skip header row
                print('Skipping header row')
                header = False
            else:
                substrs = eachLine.split(',',eachLine.count(','))
                
                species = substrs[3]
                if substrs[4].isdigit: # Check diameter is a number 
                    diameter = float(substrs[4])
                    height = float(substrs[10])
                
                    if species == 'BEECH':
                        beechDiameter.append(diameter)
                        beechHeight.append(height)
                    elif species == 'ASH':
                        ashDiameter.append(diameter)
                        ashHeight.append(height)
                    elif species == 'BIRCH':
                        birchDiameter.append(diameter)
                        birchHeight.append(height)
                    elif species == 'OAK':
                        oakDiameter.append(diameter)
                        oakHeight.append(height)
                    elif species == 'SYC':
                        sycamoreDiameter.append(diameter)
                        sycamoreHeight.append(height)
                    else:
                        otherDiameter.append(diameter)
                        otherHeight.append(height)
        
        # Convert to NumPy arrays
        beechDiameter = np.array(beechDiameter)
        ashDiameter = np.array(ashDiameter)
        birchDiameter = np.array(birchDiameter)
        oakDiameter = np.array(oakDiameter)
        sycamoreDiameter = np.array(sycamoreDiameter)
        otherDiameter = np.array(otherDiameter)
        
        beechHeight = np.array(beechHeight)
        ashHeight = np.array(ashHeight)
        birchHeight = np.array(birchHeight)
        oakHeight = np.array(oakHeight)
        sycamoreHeight = np.array(sycamoreHeight)
        otherHeight = np.array(otherHeight)
        
        # Write header line to output file
        headerLine = 'species,meanDiameter,medianDiameter,stDevDiameter,\
meanHeight,medianHeight,stDevHeight,\
meanBiomass,medianBiomass,stDevBiomass\n'
        outFile.write(headerLine)
    
        # Calculate statistics and biomass for each species and write to file
        outLine = 'Beech,' + self.createStatsLine(beechDiameter) + ',' + \
self.createStatsLine(beechHeight) + ',' + \
self.calcBiomassStatsLine(beechDiameter, beechHeight, 'BEECH') + '\n'
        outFile.write(outLine)
        outLine = 'Ash,' + self.createStatsLine(ashDiameter)  + ',' +  \
self.createStatsLine(ashHeight) + ',' + \
self.calcBiomassStatsLine(ashDiameter, ashHeight, 'ASH') + '\n'
        outFile.write(outLine)
        outLine = 'Birch,' + self.createStatsLine(birchDiameter) + ',' + \
self.createStatsLine(birchHeight) + ',' +  \
self.calcBiomassStatsLine(birchDiameter, birchHeight, 'BIRCH') + '\n'
        outFile.write(outLine)
        outLine = 'Oak,' + self.createStatsLine(oakDiameter)  + ',' + \
self.createStatsLine(oakHeight) + ',' +  \
self.calcBiomassStatsLine(oakDiameter, oakHeight, 'OAK') + '\n'
        outFile.write(outLine)
        outLine = 'Sycamore,' + self.createStatsLine(sycamoreDiameter)  + ',' + \
self.createStatsLine(sycamoreHeight) + ',' +  \
self.calcBiomassStatsLine(sycamoreDiameter, sycamoreHeight, 'SYC') + '\n'
        outFile.write(outLine)
        outLine = 'Other,' + self.createStatsLine(otherDiameter)  + ',' + \
self.createStatsLine(otherHeight) + ',' +  \
self.calcBiomassStatsLine(otherDiameter, otherHeight, 'Other') + '\n'
        outFile.write(outLine)
        
        print('Statistics written to: ' + outFileName)
                
    def createStatsLine(self, inArray):
        # Calculate statsistics for array and return output line.
        meanArray = np.mean(inArray)
        medianArray = np.median(inArray)
        stDevArray = np.std(inArray)
        
        # Create output line with stats
        statsLine = str(meanArray) + ',' + str(medianArray) + ',' + str(stDevArray) 
        return statsLine
    
    def calcBiomassStatsLine(self, inDiameterArray, inHeightArray, inSpecies):
        # Calculates biomass, calculates stats from biomass and returns output line
        biomassStatsLine = ''
        try:
            # Calculate biomass
            biomass = self.calcBiomass(inDiameterArray, inHeightArray, inSpecies)
            # Calculate stats from biomass
            meanBiomass = np.mean(biomass)
            medianBiomass = np.median(biomass)
            stDevBiomass = np.std(biomass)
            
            # Create output line
            biomassStatsLine = str(meanBiomass) + ',' + str(medianBiomass) + ',' + \
str(stDevBiomass) 
            
        except Exception:
            # Catch exception and write 'na' for all values
            biomassStatsLine = 'na,na,na'
        
        return biomassStatsLine
    
    def calcBiomass(self, inDiameterArray, inHeightArray, inSpecies):        
        if inSpecies == 'BEECH':
            a = 0.014306
            b = 0.0000748
            specificGravity = 0.56
        else: # Raise exception is species is not recognised
            raise Exception('Species not recognised')
            
        # Calcualte volume
        volume = a + ((b*(inDiameterArray)**2) * (inHeightArray**0.75))
        # Calculate biomass
        biomass = volume * specificGravity
        # Return biomass
        return biomass
        
if __name__ == '__main__':
    obj = CalculateStatistics()
    obj.run()
