#! /usr/bin/env python

#######################################
# MergeSHPfiles.py
# A python script to merge shapefiles
# Author: <YOUR NAME>
# Email: <YOUR EMAIL>
# Date: DD/MM/YYYY
# Version: 1.0
#######################################

import os

class ConvertToGeoTiff (object):

    # A function to remove the file extension ext from a file name
    def removeExtension(self, name, ext):
        # The output file name
        outName = name
        # Find how many instances of the file extension are 
        # within are in the current file name
        count = name.find(ext, 0, len(name))
        # If there are no instances of ext then -1 will be returned
        if not count == -1:
            # Replace all instances of ext with empty string.
            outName = name.replace(ext, '', name.count(ext))
        # Return output file name without ext
        return outName
    
    # A function to remove the file path a file 
    # (in this case a windows file path)
    def removeFilePathWINS(self, name):
        # Remove white space (i.e., spaces, tabs)
        name = name.strip()
        # Count the number of slashs 
        # A double slash is required because \ is a 
        # string escape charater.
        count = name.count('\\')
        # Split string into a list where slashs occurs
        nameSegments = name.split('\\', count)
        # Return the last item in the list
        return nameSegments[count]
    
    # A function to test the file extension of a file
    def checkFileExtension(self, filename, extension):
        # Boolean variable to be returned by the function
        foundExtension = False;
        # Split the filename into two parts (name + ext)
        filenamesplit = os.path.splitext(filename)
        # Get the file extension into a varaiable
        fileExtension = filenamesplit[1].strip()
        # Decide whether extensions are equal
        if(fileExtension == extension):
            foundExtension = True
        # Return result
        return foundExtension
        
    
    # A function which iterates through the directory and checks file extensions
    def findFilesExt(self, directory, extension):
        # Define a list to store output list of files
        fileList = list()
        # check whether the current directory exits
        if os.path.exists(directory):
            # check whether the given directory is a directory
            if os.path.isdir(directory):
                # list all the files within the directory
                dirFileList = os.listdir(directory)
                # Loop through the individual files within the directory
                for filename in dirFileList:
                    # Check whether file is directory or file
                    if(os.path.isdir(os.path.join(directory,filename))):
                        print(os.path.join(directory,filename) + \
                        ' is a directory and therefore ignored!')
                    elif(os.path.isfile(os.path.join(directory,filename))):
                        if(self.checkFileExtension(filename, extension)):
                            fileList.append(os.path.join(directory,filename))
                    else:
                        print(filename + ' is NOT a file or directory!')
            else:
                print(directory + ' is not a directory!')
        else:
            print(directory + ' does not exist!')
        # Return the list of files
        return fileList

    # A function to control convertion of the input files to GeoTIFF
    def convertToGeoTiff(self, inFilePath, outFilePath, inExt):
        # Get the list of files within the directory
        # provided with required extension
       fileList = self.findFilesExt(inFilePath, inExt)
       # A string for the command to be built
       command = ''
       # New output file name
       newOutFileName = ''
       # Iterate through the files.
       for file in fileList:
           # Generate new file name
           newOutFileName = self.removeExtension(self.removeFilePathWINS(file), inExt)
           # Create the command
           command = 'gdal_translate -of GTIFF ' + file + ' ' + outFilePath \
                   + newOutFileName + '.tif'
           # Execute the current command
           os.system(command)

    # A function which controls the rest of the script
    def run(self):
        # Define the input directory
        inFilePath = 'C:\\PythonCourse\\Tutorial_8\\data\\ENVI_Images\\'
        # Define the output directory
        outFilePath = 'C:\\PythonCourse\\Tutorial_8\\data\\GeoTIFF_Images\\'
        # Define the input file extension
        inputExt = '.env'
        
        # Check input file path exists and is a directory
        if not os.path.exists(inFilePath):
            print('Input Filepath does not exist')
        elif not os.path.isdir(inFilePath):
            print('Input Filepath is not a directory!')
        elif not os.path.exists(outFilePath):
            print('Output Filepath does not exist')
        elif not os.path.isdir(outFilePath):
            print('Output Filepath is not a directory!')
        else:
            # Merge the shapefiles within the filePath
            self.convertToGeoTiff(inFilePath, outFilePath, inputExt)

# The start of the code
if __name__ == '__main__':
    # Make an instance of the class
    obj = ConvertToGeoTiff()
    # Call the function run()
    obj.run()