#!/usr/bin/env python

# Import the GDAL python library
import osgeo.gdal as gdal
# Import the python Argument parser
import argparse
# Import the System library
import sys

# Define a function to read and print the images
# header information.
def printImageHeader(inputFile):
    # Open the dataset in Read Only mode
    dataset = gdal.Open(inputFile, gdal.GA_ReadOnly)
    # Check that the dataset has correctly opened
    if not dataset is None:
        # Print out the image file path.
        print(inputFile)
        # Print out the number of image bands.
        print("The image has ", dataset.RasterCount, " bands.")
        # Loop through all the image bands and print out the band name
        for n in range(dataset.RasterCount):
            print("\t", n+1, ":\t", dataset.GetRasterBand(n+1).GetDescription(), "\t")
        
        # Print out the image size in pixels
        print("Image Size [", dataset.RasterXSize, ",", dataset.RasterYSize, "]")
        
        # Get the geographic header
        # geotransform[0] = TL X Coordinate
        # geotransform[1] = X Pixel Resolution
        # geotransform[2] = X Rotation
        # geotransform[3] = TL Y Coordinate
        # geotransform[4] = Y Rotation
        # geotransform[5] = Y Pixel Resolution
        geotransform = dataset.GetGeoTransform()
        # Check that the tranformation has been correctly read.
        if not geotransform is None:
            # Print out the Origin, Pixel Size and Rotation.
            print('Origin = (',geotransform[0], ',',geotransform[3],')')
            print('Pixel Size = (',geotransform[1], ',',geotransform[5],')')
            print('Rotation = (',geotransform[2], ',',geotransform[4],')')         
        else:
            # Provide an error message is the transform has not been 
            # correctly read.
            print("Could not find a geotransform for image file ", inputFile)
    else:
        # Provide an error message if the input image file
        # could not be opened.
        print("Could not open the input image file: ", inputFile)


# This is the first part of the script to 
# be executed.
if __name__ == '__main__':
    # Create the command line options 
    # parser.
    parser = argparse.ArgumentParser()
    # Define the argument for specifying the input file.
    parser.add_argument("-i", "--input", type=str, 
                        help="Specify the input image file.")
    # Call the parser to parse the arguments.
    args = parser.parse_args()
    
    # Check that the input parameter has been specified.
    if args.input == None:
        # Print an error message if not and exit.
        print("Error: No input image file provided.")
        sys.exit()
    # Otherwise, run the function to print out the image header information.
    printImageHeader(args.input)
